<?php



require_once 'Classes/Model/Observation.php';
require_once 'Classes/Pulsar/CPSR2Combiner.php';



class ObservationData2
{
  // PUBLIC

  public function __construct( ObservationData2 $src = null )
  {
    if( $src != null )
      $this->Copy( $src );
    else
    {
      $this->observation_header = null;
      $this->data_file = null;
      $this->scrunched_file = null;
      $this->scrunching_parameters = null;
      $this->data_dir = null;
    }
  }
  
  public function Copy( ObservationData2 $src )
  {
    $this->observation_header = new Observation( $src->observation_header );
    $this->data_file = new File( $src->data_file );
    $this->scrunched_file = new File( $src->scrunched_file );
    $this->scruncing_parameters = new ScrunchingParameters( $src->scrunching_parameters );
  }
  
  public function GetOriginalFile() { return $this->priv_GetOriginalFile(); }
  
  public function GetDataFile() { return $this->priv_GetDataFile(); }

  public function GetScrunchedFile() { return $this->priv_GetScrunchedFile(); }
  
  public function SetObservationHeader( Observation $s_observation_header ) { $this->observation_header = $s_observation_header; }
  
  public function SetScrunchingParameters( ScrunchingParameters $s_params ) { $this->scrunching_parameters = $s_params; }
  
  public function GetScrunchingParameters() { return $this->scrunching_parameters; }
  
  public function SetDataDir( Dir $s_data_dir ) { $this->data_dir = $s_data_dir; }
  
  public function EstimateDownloadableSize() { return $this->priv_EstimateDownloadableSize(); }
  
  public function MakeDownloadableFile() { return $this->priv_MakeDownloadableFile(); }
  
  // PRIVATE

  private $observation_header;
  private $data_file;
  private $scrunched_file;
  private $scrunching_parameters;
  private $data_dir;
  
  private function priv_GetOriginalFile()
  {
    if( $this->observation_header == null )
      throw new GZ_Exception( 'Observation header not set', 'ObservationData::priv_GetOriginalFile' );
    
    return $this->observation_header->GetFile();
  }
  
  /**
   * This function figures out the path to the datafile, if it doesn't exist it calls MakeDataFile
   * to create it.
   **/
  private function priv_GetDataFile()
  {
    $original_file = $this->priv_GetOriginalFile();
    
    if( $this->data_dir == null )
      throw new GZ_Exception( 'Data directory not set', 'ObservationData::priv_GetDataFile' );
    
    $this->data_file = File::ConvertFullPath( $this->data_dir->GetPath() . '/' . $original_file->GetFilename() );
    
    if( !$this->data_file->Exists() )
      $this->MakeDataFile();
    
    return $this->data_file;
  }
  
  /**
   * Call GetDataFile so that we know the data file is created and have a File object pointing to it.
   * If we have scrunching parameters and scrunching is required then perform the scrunching.
   **/
  public function priv_GetScrunchedFile()
  {
    if( $this->scrunching_parameters == null )
      throw new GZ_Exception( 'Scrunching parameters not set', 'ObservationData2::priv_GetScrunchedFile' );
    
    if( !$this->scrunching_parameters->ScrunchingSet() )
      throw new GZ_Exception( 'Scrunching parameters set but no scrunching required', 'ObservationData2::priv_GetScrunchedFile' );
    
    $original_file = $this->GetOriginalFile();
    
    $scr_filename = AddPreExtension( $original_file->GetFilename(), $this->scrunching_parameters->GetTextDescription() );
    
    $this->scrunched_file = File::ConvertFullPath( $this->data_dir->GetPath() . '/' . $scr_filename );
    
    if( !$this->scrunched_file->Exists() )
      $this->MakeScrunchedFile();
    
    return $this->scrunched_file;
  }
  
  public function priv_EstimateDownloadableSize()
  {
    $nbin = $this->observation_header->GetNBin();
    $nchan = $this->observation_header->GetNChan();
    $nsub = $this->observation_header->GetNSub();
    $npol = $this->observation_header->GetNPol();
    
    $data_size = 2 * $nbin * $nchan * $nsub * $npol;
    
    if( $this->scrunching_parameters != null )
    {
      $scr_nchan = $this->scrunching_parameters->GetFScrunch();
      $scr_nsub = $this->scrunching_parameters->GetTScrunch();
      $scr_npol = $this->scrunching_parameters->GetPScrunch();
      
      $new_nchan = ceil( $nchan / $scr_nchan );
      $new_nsub = ceil( $nsub / $scr_nsub );
      $new_npol = $npol;
      if( $scr_npol )
        $new_npol = 1;
      
      $new_data_size = 2 * $nbin * $new_nchan * $new_nsub * $new_npol;
      
      $data_size = $new_data_size;
    }
    
    return $data_size;
  }
  
  /**
   * If the original file is a directory, we psradd the archives in that directory together and put
   * the result in a single file in the data directory. If the original file is a single file, we
   * create a symlink to it from the data dir.
   **/
  private function MakeDataFile()
  {
    $original_file = $this->GetOriginalFile();
    if( !$this->data_file->Exists() )
    {
      if( $original_file->IsDir() )
      {
        $src_dir = Dir::ConvertFile( $original_file );
        $dir_contents = $src_dir->ListFiles();
  
        $combiner = new CPSR2Combiner();
        foreach( $dir_contents as $next_file )
        {
          if( strpos( $next_file, '.cfb' ) !== false ||
              strpos( $next_file, '.cf' ) !== false )
          {
            $combiner->AddArchive( File::ConvertFullPath( $src_dir->GetPath() . '/' . $next_file ) );
          }
        }
        $combiner->SetDestFile( $this->data_file );
        $combiner->Execute();
      }
      else
        symlink( $original_file->GetFullPath(), $this->data_file->GetFullPath() );
    }
      
  }
  
  //! Create the scrunched file from the data_file to the scrunched_file using the scrunching_parameters
  private function MakeScrunchedFile()
  {
    $data_file = $this->priv_GetDataFile();
    
    $scruncher = new ArchiveScruncher();
    $scruncher->SetSrcFile( $this->data_file );
    $scruncher->SetDestFile( $this->scrunched_file );
    $scruncher->SetParameters( $this->scrunching_parameters );
    $scruncher->Execute();
  }
  
  //! Make the data file, and then if needed make the scrunched file
  private function priv_MakeDownloadableFile()
  {
    $this->GetDataFile();
    if( $this->scrunching_parameters != null )
      $this->MakeScrunchedFile();
  }
};